@@ -18,7 +18,7 @@
 if {[info commands package] == ""} {
     error "version mismatch: library\nscripts expect Tcl version 7.5b1 or later but the loaded version is\nonly [info patchlevel]"
 }
-package require -exact Tcl 8.6.16
+package require -exact Tcl 8.6.17
 
 # Compute the auto path to use in this interpreter.
 # The values on the path come from several locations:
@@ -652,8 +652,10 @@ proc auto_execok name {
     }
     set auto_execs($name) ""
 
-    set shellBuiltins [list assoc cls copy date del dir echo erase exit ftype \
-	    md mkdir mklink move rd ren rename rmdir start time type ver vol]
+    set shellBuiltins [list assoc call cd cls color copy date del dir echo \
+                           erase exit ftype for if md mkdir mklink move path \
+                           pause prompt rd ren rename rmdir set start time \
+                           title type ver vol]
     if {[info exists env(PATHEXT)]} {
 	# Add an initial ; to have the {} extension check first.
 	set execExtensions [split ";$env(PATHEXT)" ";"]
